@@ -18,6 +18,7 @@
  * Copyright (C) 2016 Arseniy Lartsev <arseniy@alumni.chalmers.se>
  * Copyright (C) 2016, Hanno Meyer-Thurow <h.mth@web.de>
  * Copyright (C) 2017, Oliver Sander <oliver.sander@tu-dresden.de>
+ * Copyright (C) 2017 Adrian Johnson <ajohnson@redneon.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -307,7 +308,7 @@ QImage Page::renderToImage(double xres, double yres, int x, int y, int w, int h,
 #if defined(HAVE_SPLASH)
       SplashColor bgColor;
       GBool overprintPreview = gFalse;
-#if SPLASH_CMYK
+#ifdef SPLASH_CMYK
       overprintPreview = m_page->parentDoc->m_hints & Document::OverprintPreview ? gTrue : gFalse;
       if (overprintPreview)
       {
@@ -340,7 +341,7 @@ QImage Page::renderToImage(double xres, double yres, int x, int y, int w, int h,
       }
 
       SplashColorMode colorMode = splashModeXBGR8;
-#if SPLASH_CMYK
+#ifdef SPLASH_CMYK
       if (overprintPreview) colorMode = splashModeDeviceN8;
 #endif
 
