@@ -15,10 +15,10 @@
 // under GPL version 2 or later
 //
 // Copyright (C) 2006 Dom Lachowicz <cinamod@hotmail.com>
-// Copyright (C) 2007-2010, 2012 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2007-2010, 2012, 2016, 2017 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2010 Hib Eris <hib@hiberis.nl>
 // Copyright (C) 2011 Vittal Aithal <vittal.aithal@cognidox.com>
-// Copyright (C) 2012, 2013, 2016 Adrian Johnson <ajohnson@redneon.com>
+// Copyright (C) 2012, 2013, 2016, 2017 Adrian Johnson <ajohnson@redneon.com>
 // Copyright (C) 2012 Fabio D'Urso <fabiodurso@hotmail.it>
 // Copyright (C) 2013 Adrian Perez de Castro <aperez@igalia.com>
 // Copyright (C) 2013 Suzuki Toshiya <mpsuzuki@hiroshima-u.ac.jp>
@@ -36,6 +36,7 @@
 #include <string.h>
 #include <time.h>
 #include <math.h>
+#include <map>
 #include "parseargs.h"
 #include "printencodings.h"
 #include "goo/GooString.h"
@@ -61,13 +62,6 @@
 #include "StructTreeRoot.h"
 #include "StructElement.h"
 
-static void printInfoString(Dict *infoDict, const char *key, const char *text,
-			    UnicodeMap *uMap);
-static void printInfoDate(Dict *infoDict, const char *key, const char *text);
-static void printISODate(Dict *infoDict, const char *key, const char *text);
-static void printBox(const char *text, PDFRectangle *box);
-static void printStruct(const StructElement *element, unsigned indent = 0);
-static void printIndent(unsigned level);
 
 static int firstPage = 1;
 static int lastPage = 0;
@@ -84,6 +78,7 @@ static GBool printHelp = gFalse;
 static GBool printEnc = gFalse;
 static GBool printStructure = gFalse;
 static GBool printStructureText = gFalse;
+static GBool printDests = gFalse;
 
 static const ArgDesc argDesc[] = {
   {"-f",      argInt,      &firstPage,        0,
@@ -104,6 +99,8 @@ static const ArgDesc argDesc[] = {
    "print the dates in ISO-8601 format"},
   {"-rawdates", argFlag,   &rawDates,         0,
    "print the undecoded date strings directly from the PDF file"},
+  {"-dests",     argFlag,  &printDests,       0,
+   "print all named destinations in the PDF"},
   {"-enc",    argString,   textEncName,    sizeof(textEncName),
    "output text encoding name"},
   {"-listenc",argFlag,     &printEnc,      0,
@@ -125,340 +122,15 @@ static const ArgDesc argDesc[] = {
   {NULL}
 };
 
-int main(int argc, char *argv[]) {
-  PDFDoc *doc;
-  GooString *fileName;
-  GooString *ownerPW, *userPW;
-  UnicodeMap *uMap;
-  Page *page;
-  Object info;
-  char buf[256];
-  double w, h, wISO, hISO;
-  FILE *f;
-  GooString *metadata;
-  GBool ok;
-  int exitCode;
-  int pg, i;
-  GBool multiPage;
-  int r;
-
-  exitCode = 99;
-
-  // parse args
-  ok = parseArgs(argDesc, &argc, argv);
-  if (!ok || (argc != 2 && !printEnc) || printVersion || printHelp) {
-    fprintf(stderr, "pdfinfo version %s\n", PACKAGE_VERSION);
-    fprintf(stderr, "%s\n", popplerCopyright);
-    fprintf(stderr, "%s\n", xpdfCopyright);
-    if (!printVersion) {
-      printUsage("pdfinfo", "<PDF-file>", argDesc);
-    }
-    if (printVersion || printHelp)
-      exitCode = 0;
-    goto err0;
-  }
-
-  if (printStructureText)
-    printStructure = gTrue;
-
-  // read config file
-  globalParams = new GlobalParams();
-
-  if (printEnc) {
-    printEncodings();
-    delete globalParams;
-    exitCode = 0;
-    goto err0;
-  }
-
-  fileName = new GooString(argv[1]);
-
-  if (textEncName[0]) {
-    globalParams->setTextEncoding(textEncName);
-  }
-
-  // get mapping to output encoding
-  if (!(uMap = globalParams->getTextEncoding())) {
-    error(errCommandLine, -1, "Couldn't get text encoding");
-    delete fileName;
-    goto err1;
-  }
-
-  // open PDF file
-  if (ownerPassword[0] != '\001') {
-    ownerPW = new GooString(ownerPassword);
-  } else {
-    ownerPW = NULL;
-  }
-  if (userPassword[0] != '\001') {
-    userPW = new GooString(userPassword);
-  } else {
-    userPW = NULL;
-  }
-
-  if (fileName->cmp("-") == 0) {
-      delete fileName;
-      fileName = new GooString("fd://0");
-  }
-
-  doc = PDFDocFactory().createPDFDoc(*fileName, ownerPW, userPW);
-
-  if (userPW) {
-    delete userPW;
-  }
-  if (ownerPW) {
-    delete ownerPW;
-  }
-  if (!doc->isOk()) {
-    exitCode = 1;
-    goto err2;
-  }
-
-  // get page range
-  if (firstPage < 1) {
-    firstPage = 1;
-  }
-  if (lastPage == 0) {
-    multiPage = gFalse;
-    lastPage = 1;
-  } else {
-    multiPage = gTrue;
-  }
-  if (lastPage < 1 || lastPage > doc->getNumPages()) {
-    lastPage = doc->getNumPages();
-  }
-  if (lastPage < firstPage) {
-    error(errCommandLine, -1,
-          "Wrong page range given: the first page ({0:d}) can not be after the last page ({1:d}).",
-          firstPage, lastPage);
-    goto err2;
-  }
-
-  // print doc info
-  doc->getDocInfo(&info);
-  if (info.isDict()) {
-    printInfoString(info.getDict(), "Title",        "Title:          ", uMap);
-    printInfoString(info.getDict(), "Subject",      "Subject:        ", uMap);
-    printInfoString(info.getDict(), "Keywords",     "Keywords:       ", uMap);
-    printInfoString(info.getDict(), "Author",       "Author:         ", uMap);
-    printInfoString(info.getDict(), "Creator",      "Creator:        ", uMap);
-    printInfoString(info.getDict(), "Producer",     "Producer:       ", uMap);
-    if (isoDates) {
-      printISODate(info.getDict(),   "CreationDate", "CreationDate:   ");
-      printISODate(info.getDict(),   "ModDate",      "ModDate:        ");
-    } else if (rawDates) {
-      printInfoString(info.getDict(), "CreationDate", "CreationDate:   ",
-		      uMap);
-      printInfoString(info.getDict(), "ModDate",      "ModDate:        ",
-		      uMap);
-    } else {
-      printInfoDate(info.getDict(),   "CreationDate", "CreationDate:   ");
-      printInfoDate(info.getDict(),   "ModDate",      "ModDate:        ");
-    }
-  }
-  info.free();
-
-  // print tagging info
-   printf("Tagged:         %s\n",
-	  (doc->getCatalog()->getMarkInfo() & Catalog::markInfoMarked) ? "yes" : "no");
-   printf("UserProperties: %s\n",
-	  (doc->getCatalog()->getMarkInfo() & Catalog::markInfoUserProperties) ? "yes" : "no");
-   printf("Suspects:       %s\n",
-	  (doc->getCatalog()->getMarkInfo() & Catalog::markInfoSuspects) ? "yes" : "no");
-
-  // print form info
-  switch (doc->getCatalog()->getFormType())
-  {
-    case Catalog::NoForm:
-      printf("Form:           none\n");
-      break;
-    case Catalog::AcroForm:
-      printf("Form:           AcroForm\n");
-      break;
-    case Catalog::XfaForm:
-      printf("Form:           XFA\n");
-      break;
-  }
-
-  // print javascript info
-  {
-    JSInfo jsInfo(doc, firstPage - 1);
-    jsInfo.scanJS(lastPage - firstPage + 1);
-    printf("JavaScript:     %s\n", jsInfo.containsJS() ? "yes" : "no");
-  }
-
-  // print page count
-  printf("Pages:          %d\n", doc->getNumPages());
-
-  // print encryption info
-  printf("Encrypted:      ");
-  if (doc->isEncrypted()) {
-    Guchar *fileKey;
-    CryptAlgorithm encAlgorithm;
-    int keyLength;
-    doc->getXRef()->getEncryptionParameters(&fileKey, &encAlgorithm, &keyLength);
-
-    const char *encAlgorithmName = "unknown";
-    switch (encAlgorithm)
-    {
-      case cryptRC4:
-	encAlgorithmName = "RC4";
-	break;
-      case cryptAES:
-	encAlgorithmName = "AES";
-	break;
-      case cryptAES256:
-	encAlgorithmName = "AES-256";
-	break;
-    }
-
-    printf("yes (print:%s copy:%s change:%s addNotes:%s algorithm:%s)\n",
-	   doc->okToPrint(gTrue) ? "yes" : "no",
-	   doc->okToCopy(gTrue) ? "yes" : "no",
-	   doc->okToChange(gTrue) ? "yes" : "no",
-	   doc->okToAddNotes(gTrue) ? "yes" : "no",
-	   encAlgorithmName);
-  } else {
-    printf("no\n");
-  }
-
-  // print page size
-  for (pg = firstPage; pg <= lastPage; ++pg) {
-    w = doc->getPageCropWidth(pg);
-    h = doc->getPageCropHeight(pg);
-    if (multiPage) {
-      printf("Page %4d size: %g x %g pts", pg, w, h);
-    } else {
-      printf("Page size:      %g x %g pts", w, h);
-    }
-    if ((fabs(w - 612) < 0.1 && fabs(h - 792) < 0.1) ||
-	(fabs(w - 792) < 0.1 && fabs(h - 612) < 0.1)) {
-      printf(" (letter)");
-    } else {
-      hISO = sqrt(sqrt(2.0)) * 7200 / 2.54;
-      wISO = hISO / sqrt(2.0);
-      for (i = 0; i <= 6; ++i) {
-	if ((fabs(w - wISO) < 1 && fabs(h - hISO) < 1) ||
-	    (fabs(w - hISO) < 1 && fabs(h - wISO) < 1)) {
-	  printf(" (A%d)", i);
-	  break;
-	}
-	hISO = wISO;
-	wISO /= sqrt(2.0);
-      }
-    }
-    printf("\n");
-    r = doc->getPageRotate(pg);
-    if (multiPage) {
-      printf("Page %4d rot:  %d\n", pg, r);
-    } else {
-      printf("Page rot:       %d\n", r);
-    }
-  } 
-
-  // print the boxes
-  if (printBoxes) {
-    if (multiPage) {
-      for (pg = firstPage; pg <= lastPage; ++pg) {
-	page = doc->getPage(pg);
-	if (!page) {
-          error(errSyntaxError, -1, "Failed to print boxes for page {0:d}", pg);
-	  continue;
-	}
-	sprintf(buf, "Page %4d MediaBox: ", pg);
-	printBox(buf, page->getMediaBox());
-	sprintf(buf, "Page %4d CropBox:  ", pg);
-	printBox(buf, page->getCropBox());
-	sprintf(buf, "Page %4d BleedBox: ", pg);
-	printBox(buf, page->getBleedBox());
-	sprintf(buf, "Page %4d TrimBox:  ", pg);
-	printBox(buf, page->getTrimBox());
-	sprintf(buf, "Page %4d ArtBox:   ", pg);
-	printBox(buf, page->getArtBox());
-      }
-    } else {
-      page = doc->getPage(firstPage);
-      if (!page) {
-        error(errSyntaxError, -1, "Failed to print boxes for page {0:d}", firstPage);
-      } else {
-        printBox("MediaBox:       ", page->getMediaBox());
-        printBox("CropBox:        ", page->getCropBox());
-        printBox("BleedBox:       ", page->getBleedBox());
-        printBox("TrimBox:        ", page->getTrimBox());
-        printBox("ArtBox:         ", page->getArtBox());
-      }
-    }
-  }
-
-  // print file size
-#ifdef VMS
-  f = fopen(fileName->getCString(), "rb", "ctx=stm");
-#else
-  f = fopen(fileName->getCString(), "rb");
-#endif
-  if (f) {
-    Gfseek(f, 0, SEEK_END);
-    printf("File size:      %lld bytes\n", (long long)Gftell(f));
-    fclose(f);
-  }
-
-  // print linearization info
-  printf("Optimized:      %s\n", doc->isLinearized() ? "yes" : "no");
-
-  // print PDF version
-  printf("PDF version:    %d.%d\n", doc->getPDFMajorVersion(), doc->getPDFMinorVersion());
-
-  // print the metadata
-  if (printMetadata && (metadata = doc->readMetadata())) {
-    fputs("Metadata:\n", stdout);
-    fputs(metadata->getCString(), stdout);
-    fputc('\n', stdout);
-    delete metadata;
-  }
-
-  // print javascript
-  if (printJS) {
-    JSInfo jsInfo(doc, firstPage - 1);
-    fputs("\n", stdout);
-    jsInfo.scanJS(lastPage - firstPage + 1, stdout, uMap);
-  }
-
-  // print the structure
-  const StructTreeRoot *structTree;
-  if (printStructure && (structTree = doc->getCatalog()->getStructTreeRoot())) {
-    fputs("Structure:\n", stdout);
-    for (unsigned i = 0; i < structTree->getNumChildren(); i++) {
-      printStruct(structTree->getChild(i), 1);
-    }
-  }
-
-  exitCode = 0;
-
-  // clean up
- err2:
-  uMap->decRefCnt();
-  delete doc;
-  delete fileName;
- err1:
-  delete globalParams;
- err0:
-
-  // check for memory leaks
-  Object::memCheck(stderr);
-  gMemReport(stderr);
-
-  return exitCode;
-}
-
 static void printInfoString(Dict *infoDict, const char *key, const char *text,
 			    UnicodeMap *uMap) {
-  Object obj;
   GooString *s1;
   Unicode *u;
   char buf[8];
   int i, n, len;
 
-  if (infoDict->lookup(key, &obj)->isString()) {
+  Object obj = infoDict->lookup(key);
+  if (obj.isString()) {
     fputs(text, stdout);
     s1 = obj.getString();
     len = TextStringToUCS4(s1, &u);
@@ -469,11 +141,9 @@ static void printInfoString(Dict *infoDict, const char *key, const char *text,
     gfree(u);
     fputc('\n', stdout);
   }
-  obj.free();
 }
 
 static void printInfoDate(Dict *infoDict, const char *key, const char *text) {
-  Object obj;
   char *s;
   int year, mon, day, hour, min, sec, tz_hour, tz_minute;
   char tz;
@@ -481,7 +151,8 @@ static void printInfoDate(Dict *infoDict, const char *key, const char *text) {
   time_t time;
   char buf[256];
 
-  if (infoDict->lookup(key, &obj)->isString()) {
+  Object obj = infoDict->lookup(key);
+  if (obj.isString()) {
     fputs(text, stdout);
     s = obj.getString()->getCString();
     // TODO do something with the timezone info
@@ -513,17 +184,16 @@ static void printInfoDate(Dict *infoDict, const char *key, const char *text) {
     }
     fputc('\n', stdout);
   }
-  obj.free();
 }
 
-void printISODate(Dict *infoDict, const char *key, const char *text)
+static void printISODate(Dict *infoDict, const char *key, const char *text)
 {
-  Object obj;
   char *s;
   int year, mon, day, hour, min, sec, tz_hour, tz_minute;
   char tz;
 
-  if (infoDict->lookup(key, &obj)->isString()) {
+  Object obj = infoDict->lookup(key);
+  if (obj.isString()) {
     fputs(text, stdout);
     s = obj.getString()->getCString();
     if ( parseDateString( s, &year, &mon, &day, &hour, &min, &sec, &tz, &tz_hour, &tz_minute ) ) {
@@ -540,7 +210,6 @@ void printISODate(Dict *infoDict, const char *key, const char *text)
     }
     fputc('\n', stdout);
   }
-  obj.free();
 }
 
 static void printBox(const char *text, PDFRectangle *box) {
@@ -620,3 +289,477 @@ static void printStruct(const StructElement *element, unsigned indent) {
       }
   }
 }
+
+struct RefCompare {
+  bool operator() (const Ref& lhs, const Ref& rhs) const {
+    return lhs.num < rhs.num;
+  }
+};
+
+struct GooStringCompare {
+  bool operator() (GooString* lhs, GooString* rhs) const {
+    return lhs->cmp(const_cast<GooString*>(rhs)) < 0;
+  }
+};
+
+static void printLinkDest(LinkDest *dest) {
+  GooString s;
+
+  switch (dest->getKind()) {
+    case destXYZ:
+      s.append("[ XYZ ");
+      if (dest->getChangeLeft()) {
+	s.appendf("{0:4.0g} ", dest->getLeft());
+      } else {
+	s.append("null ");
+      }
+      if (dest->getChangeTop()) {
+	s.appendf("{0:4.0g} ", dest->getTop());
+      } else {
+	s.append("null ");
+      }
+      if (dest->getChangeZoom()) {
+	s.appendf("{0:4.2f} ", dest->getZoom());
+      } else {
+	s.append("null ");
+      }
+      break;
+    case destFit:
+      s.append("[ Fit ");
+      break;
+    case destFitH:
+      if (dest->getChangeTop()) {
+	s.appendf("[ FitH {0:4.0g} ", dest->getTop());
+      } else {
+	s.append("[ FitH null ");
+      }
+      break;
+    case destFitV:
+      if (dest->getChangeLeft()) {
+	s.appendf("[ FitV {0:4.0g} ", dest->getLeft());
+      } else {
+	s.append("[ FitV null ");
+      }
+      break;
+    case destFitR:
+      s.appendf("[ FitR {0:4.0g} {1:4.0g} {2:4.0g} {3:4.0g} ",
+	      dest->getLeft(),
+	      dest->getBottom(),
+	      dest->getRight(),
+	      dest->getTop());
+      break;
+    case destFitB:
+      s.append("[ FitB ");
+      break;
+    case destFitBH:
+      if (dest->getChangeTop()) {
+	s.appendf("[ FitBH {0:4.0g} ", dest->getTop());
+      } else {
+	s.append("[ FitBH null ");
+      }
+      break;
+    case destFitBV:
+      if (dest->getChangeLeft()) {
+	s.appendf("[ FitBV {0:4.0g} ", dest->getLeft());
+      } else {
+	s.append("[ FitBV null ");
+      }
+      break;
+  }
+
+  s.append("                                ");
+  s.setChar(26, ']');
+  s.setChar(27, '\0');
+  printf("%s", s.getCString());
+}
+
+static void printDestinations(PDFDoc *doc, UnicodeMap *uMap) {
+  std::map<Ref,std::map<GooString*,LinkDest*,GooStringCompare>, RefCompare > map;
+
+  int numDests = doc->getCatalog()->numDestNameTree();
+  for (int i = 0; i < numDests; i++) {
+    GooString *name = new GooString(doc->getCatalog()->getDestNameTreeName(i));
+    LinkDest *dest = doc->getCatalog()->getDestNameTreeDest(i);
+    if (dest && dest->isPageRef()) {
+      map[dest->getPageRef()].insert(std::make_pair(name, dest));
+    } else {
+      delete name;
+      delete dest;
+    }
+  }
+
+  numDests = doc->getCatalog()->numDests();
+  for (int i = 0; i < numDests; i++) {
+    GooString *name = new GooString(doc->getCatalog()->getDestsName(i));
+    LinkDest *dest = doc->getCatalog()->getDestsDest(i);
+    if (dest && dest->isPageRef()) {
+      map[dest->getPageRef()].insert(std::make_pair(name, dest));
+    } else {
+      delete name;
+      delete dest;
+    }
+  }
+
+  printf("Page  Destination                 Name\n");
+  for (int i = firstPage; i <= lastPage; i++) {
+    Ref *ref = doc->getCatalog()->getPageRef(i);
+    if (ref) {
+      auto pageDests = map.find(*ref);
+      if (pageDests != map.end()) {
+	for (auto& it: pageDests->second) {
+	  printf("%4d ", i);
+	  printLinkDest(it.second);
+	  printf(" \"");
+	  Unicode *u;
+	  char buf[8];
+	  int n, len;
+	  len = TextStringToUCS4(it.first, &u);
+	  for (int i = 0; i < len; i++) {
+	    n = uMap->mapUnicode(u[i], buf, sizeof(buf));
+	    fwrite(buf, 1, n, stdout);
+	  }
+	  gfree(u);
+	  printf("\"\n");
+	  delete it.first;
+	  delete it.second;
+	}
+      }
+    }
+  }
+}
+
+static void printInfo(PDFDoc *doc, UnicodeMap *uMap, long long filesize, GBool multiPage) {
+  Page *page;
+  char buf[256];
+  double w, h, wISO, hISO;
+  int pg, i;
+  int r;
+
+  // print doc info
+  Object info = doc->getDocInfo();
+  if (info.isDict()) {
+    printInfoString(info.getDict(), "Title",        "Title:          ", uMap);
+    printInfoString(info.getDict(), "Subject",      "Subject:        ", uMap);
+    printInfoString(info.getDict(), "Keywords",     "Keywords:       ", uMap);
+    printInfoString(info.getDict(), "Author",       "Author:         ", uMap);
+    printInfoString(info.getDict(), "Creator",      "Creator:        ", uMap);
+    printInfoString(info.getDict(), "Producer",     "Producer:       ", uMap);
+    if (isoDates) {
+      printISODate(info.getDict(),   "CreationDate", "CreationDate:   ");
+      printISODate(info.getDict(),   "ModDate",      "ModDate:        ");
+    } else if (rawDates) {
+      printInfoString(info.getDict(), "CreationDate", "CreationDate:   ",
+		      uMap);
+      printInfoString(info.getDict(), "ModDate",      "ModDate:        ",
+		      uMap);
+    } else {
+      printInfoDate(info.getDict(),   "CreationDate", "CreationDate:   ");
+      printInfoDate(info.getDict(),   "ModDate",      "ModDate:        ");
+    }
+  }
+
+  // print tagging info
+   printf("Tagged:         %s\n",
+	  (doc->getCatalog()->getMarkInfo() & Catalog::markInfoMarked) ? "yes" : "no");
+   printf("UserProperties: %s\n",
+	  (doc->getCatalog()->getMarkInfo() & Catalog::markInfoUserProperties) ? "yes" : "no");
+   printf("Suspects:       %s\n",
+	  (doc->getCatalog()->getMarkInfo() & Catalog::markInfoSuspects) ? "yes" : "no");
+
+  // print form info
+  switch (doc->getCatalog()->getFormType())
+  {
+    case Catalog::NoForm:
+      printf("Form:           none\n");
+      break;
+    case Catalog::AcroForm:
+      printf("Form:           AcroForm\n");
+      break;
+    case Catalog::XfaForm:
+      printf("Form:           XFA\n");
+      break;
+  }
+
+  // print javascript info
+  {
+    JSInfo jsInfo(doc, firstPage - 1);
+    jsInfo.scanJS(lastPage - firstPage + 1);
+    printf("JavaScript:     %s\n", jsInfo.containsJS() ? "yes" : "no");
+  }
+
+  // print page count
+  printf("Pages:          %d\n", doc->getNumPages());
+
+  // print encryption info
+  printf("Encrypted:      ");
+  if (doc->isEncrypted()) {
+    Guchar *fileKey;
+    CryptAlgorithm encAlgorithm;
+    int keyLength;
+    doc->getXRef()->getEncryptionParameters(&fileKey, &encAlgorithm, &keyLength);
+
+    const char *encAlgorithmName = "unknown";
+    switch (encAlgorithm)
+    {
+      case cryptRC4:
+	encAlgorithmName = "RC4";
+	break;
+      case cryptAES:
+	encAlgorithmName = "AES";
+	break;
+      case cryptAES256:
+	encAlgorithmName = "AES-256";
+	break;
+      case cryptNone:
+	break;
+    }
+
+    printf("yes (print:%s copy:%s change:%s addNotes:%s algorithm:%s)\n",
+	   doc->okToPrint(gTrue) ? "yes" : "no",
+	   doc->okToCopy(gTrue) ? "yes" : "no",
+	   doc->okToChange(gTrue) ? "yes" : "no",
+	   doc->okToAddNotes(gTrue) ? "yes" : "no",
+	   encAlgorithmName);
+  } else {
+    printf("no\n");
+  }
+
+  // print page size
+  for (pg = firstPage; pg <= lastPage; ++pg) {
+    w = doc->getPageCropWidth(pg);
+    h = doc->getPageCropHeight(pg);
+    if (multiPage) {
+      printf("Page %4d size: %g x %g pts", pg, w, h);
+    } else {
+      printf("Page size:      %g x %g pts", w, h);
+    }
+    if ((fabs(w - 612) < 0.1 && fabs(h - 792) < 0.1) ||
+	(fabs(w - 792) < 0.1 && fabs(h - 612) < 0.1)) {
+      printf(" (letter)");
+    } else {
+      hISO = sqrt(sqrt(2.0)) * 7200 / 2.54;
+      wISO = hISO / sqrt(2.0);
+      for (i = 0; i <= 6; ++i) {
+	if ((fabs(w - wISO) < 1 && fabs(h - hISO) < 1) ||
+	    (fabs(w - hISO) < 1 && fabs(h - wISO) < 1)) {
+	  printf(" (A%d)", i);
+	  break;
+	}
+	hISO = wISO;
+	wISO /= sqrt(2.0);
+      }
+    }
+    printf("\n");
+    r = doc->getPageRotate(pg);
+    if (multiPage) {
+      printf("Page %4d rot:  %d\n", pg, r);
+    } else {
+      printf("Page rot:       %d\n", r);
+    }
+  }
+
+  // print the boxes
+  if (printBoxes) {
+    if (multiPage) {
+      for (pg = firstPage; pg <= lastPage; ++pg) {
+	page = doc->getPage(pg);
+	if (!page) {
+          error(errSyntaxError, -1, "Failed to print boxes for page {0:d}", pg);
+	  continue;
+	}
+	sprintf(buf, "Page %4d MediaBox: ", pg);
+	printBox(buf, page->getMediaBox());
+	sprintf(buf, "Page %4d CropBox:  ", pg);
+	printBox(buf, page->getCropBox());
+	sprintf(buf, "Page %4d BleedBox: ", pg);
+	printBox(buf, page->getBleedBox());
+	sprintf(buf, "Page %4d TrimBox:  ", pg);
+	printBox(buf, page->getTrimBox());
+	sprintf(buf, "Page %4d ArtBox:   ", pg);
+	printBox(buf, page->getArtBox());
+      }
+    } else {
+      page = doc->getPage(firstPage);
+      if (!page) {
+        error(errSyntaxError, -1, "Failed to print boxes for page {0:d}", firstPage);
+      } else {
+        printBox("MediaBox:       ", page->getMediaBox());
+        printBox("CropBox:        ", page->getCropBox());
+        printBox("BleedBox:       ", page->getBleedBox());
+        printBox("TrimBox:        ", page->getTrimBox());
+        printBox("ArtBox:         ", page->getArtBox());
+      }
+    }
+  }
+
+  // print file size
+  printf("File size:      %lld bytes\n", filesize);
+
+  // print linearization info
+  printf("Optimized:      %s\n", doc->isLinearized() ? "yes" : "no");
+
+  // print PDF version
+  printf("PDF version:    %d.%d\n", doc->getPDFMajorVersion(), doc->getPDFMinorVersion());
+}
+
+int main(int argc, char *argv[]) {
+  PDFDoc *doc;
+  GooString *fileName;
+  GooString *ownerPW, *userPW;
+  UnicodeMap *uMap;
+  Object info;
+  FILE *f;
+  GBool ok;
+  int exitCode;
+  GBool multiPage;
+
+  exitCode = 99;
+
+  // parse args
+  ok = parseArgs(argDesc, &argc, argv);
+  if (!ok || (argc != 2 && !printEnc) || printVersion || printHelp) {
+    fprintf(stderr, "pdfinfo version %s\n", PACKAGE_VERSION);
+    fprintf(stderr, "%s\n", popplerCopyright);
+    fprintf(stderr, "%s\n", xpdfCopyright);
+    if (!printVersion) {
+      printUsage("pdfinfo", "<PDF-file>", argDesc);
+    }
+    if (printVersion || printHelp)
+      exitCode = 0;
+    goto err0;
+  }
+
+  if (printStructureText)
+    printStructure = gTrue;
+
+  // read config file
+  globalParams = new GlobalParams();
+
+  if (printEnc) {
+    printEncodings();
+    delete globalParams;
+    exitCode = 0;
+    goto err0;
+  }
+
+  fileName = new GooString(argv[1]);
+
+  if (textEncName[0]) {
+    globalParams->setTextEncoding(textEncName);
+  }
+
+  // get mapping to output encoding
+  if (!(uMap = globalParams->getTextEncoding())) {
+    error(errCommandLine, -1, "Couldn't get text encoding");
+    delete fileName;
+    goto err1;
+  }
+
+  // open PDF file
+  if (ownerPassword[0] != '\001') {
+    ownerPW = new GooString(ownerPassword);
+  } else {
+    ownerPW = NULL;
+  }
+  if (userPassword[0] != '\001') {
+    userPW = new GooString(userPassword);
+  } else {
+    userPW = NULL;
+  }
+
+  if (fileName->cmp("-") == 0) {
+      delete fileName;
+      fileName = new GooString("fd://0");
+  }
+
+  doc = PDFDocFactory().createPDFDoc(*fileName, ownerPW, userPW);
+
+  if (userPW) {
+    delete userPW;
+  }
+  if (ownerPW) {
+    delete ownerPW;
+  }
+  if (!doc->isOk()) {
+    exitCode = 1;
+    goto err2;
+  }
+
+  // get page range
+  if (firstPage < 1) {
+    firstPage = 1;
+  }
+  if (lastPage == 0) {
+    multiPage = gFalse;
+  } else {
+    multiPage = gTrue;
+  }
+  if (lastPage < 1 || lastPage > doc->getNumPages()) {
+    lastPage = doc->getNumPages();
+  }
+  if (lastPage < firstPage) {
+    error(errCommandLine, -1,
+          "Wrong page range given: the first page ({0:d}) can not be after the last page ({1:d}).",
+          firstPage, lastPage);
+    goto err2;
+  }
+
+  if (printMetadata) {
+    // print the metadata
+    GooString *metadata = doc->readMetadata();
+    if (metadata) {
+      fputs(metadata->getCString(), stdout);
+      fputc('\n', stdout);
+      delete metadata;
+    }
+  } else if (printJS) {
+    // print javascript
+    JSInfo jsInfo(doc, firstPage - 1);
+    jsInfo.scanJS(lastPage - firstPage + 1, stdout, uMap);
+  } else if (printStructure || printStructureText) {
+    // print structure
+    const StructTreeRoot *structTree = doc->getCatalog()->getStructTreeRoot();
+    if (structTree) {
+      for (unsigned i = 0; i < structTree->getNumChildren(); i++) {
+	printStruct(structTree->getChild(i), 0);
+      }
+    }
+  } else if (printDests) {
+    printDestinations(doc, uMap);
+  } else {
+    // print info
+    long long filesize = 0;
+
+#ifdef VMS
+    f = fopen(fileName->getCString(), "rb", "ctx=stm");
+#else
+    f = fopen(fileName->getCString(), "rb");
+#endif
+    if (f) {
+      Gfseek(f, 0, SEEK_END);
+      filesize = Gftell(f);
+      fclose(f);
+    }
+
+    if (multiPage == gFalse)
+      lastPage = 1;
+
+    printInfo(doc, uMap, filesize, multiPage);
+  }
+  exitCode = 0;
+
+  // clean up
+ err2:
+  uMap->decRefCnt();
+  delete doc;
+  delete fileName;
+ err1:
+  delete globalParams;
+ err0:
+
+  // check for memory leaks
+  Object::memCheck(stderr);
+  gMemReport(stderr);
+
+  return exitCode;
+}
